package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: EngineSink.java 61 2008-07-01 00:27:28Z toni $

import java.io.PrintStream;
import java.io.Reader;
import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptException;

/**
 * Intercepts ScriptEngine's methods and dumps their information to console.
 * @see SandboxImpl#getEngine
 */
class EngineSink implements ScriptEngine {
    
    private final ScriptEngine e;
    private final PrintStream s;
    
    {
        s = System.err;
    }
    
    public EngineSink(final ScriptEngine actualEngine) {
        e = actualEngine;
    }

    public Object eval(String script, ScriptContext context) throws ScriptException {
        return e.eval(script, context);
    }

    public Object eval(Reader reader, ScriptContext context) throws ScriptException {
        return e.eval(reader, context);
    }

    public Object eval(String script) throws ScriptException {
        s.printf("//[D] eval(String):\n%s//[EOS]\n", script);
        return e.eval(script);
    }

    public Object eval(Reader reader) throws ScriptException {
        return e.eval(reader);
    }

    public Object eval(String script, Bindings n) throws ScriptException {
        return e.eval(script, n);
    }

    public Object eval(Reader reader, Bindings n) throws ScriptException {
        return e.eval(reader, n);
    }

    public void put(String key, Object value) {
        s.printf("//[D] put('%s', %s)\n", key, value);
        e.put(key, value);
    }

    public Object get(String key) {
        return e.get(key);
    }

    public Bindings getBindings(int scope) {
        return e.getBindings(scope);
    }

    public void setBindings(Bindings bindings, int scope) {
        s.printf("//[D] setBindings() scope=%s:\n",
                    (scope==ScriptContext.ENGINE_SCOPE) ? "ENGINE_SCOPE" : "GLOBAL_SCOPE");
        for (final String key : bindings.keySet()) {
            s.printf("  %s=%s\n", key, bindings.get(key));
        }
        s.println("//[EOL]");
        e.setBindings(bindings, scope);
    }

    public Bindings createBindings() {
        return e.createBindings();
    }

    public ScriptContext getContext() {
        return e.getContext();
    }

    public void setContext(ScriptContext context) {
        e.setContext(context);
    }

    public ScriptEngineFactory getFactory() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
