package net.outlyer.plugins;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: SandboxAccessorImpl.java 61 2008-07-01 00:27:28Z toni $

import java.lang.reflect.Field;
import java.util.concurrent.Callable;

/**
 * Default implementation of a {@see SandboxAccessor}.
 */
public abstract class SandboxAccessorImpl implements SandboxAccessor {
    public Object sandboxGetter;

    public static Sandbox getSandbox(final SandboxAccessor sa) {
        try {
            // Retrieve the field...
            final Field gS = sa.getClass().getField("sandboxGetter");
            // Evaluate as a callable
            return ((Callable<Sandbox>)gS.get(sa)).call();
        }
        catch (final NoSuchFieldException e) {
            throw new IllegalStateException(sa.getClass()+" hasn't been correctly initialised", e);
        }
        catch (final Exception e) {
            throw new IllegalStateException("getSandbox produced an exception ("+
                                  e.getClass().getSimpleName()+"),"+
                                  "for object "+sa+" which is not expected!: " +
                                  e.getMessage(), e);
        }
    }

    public final Sandbox getSandbox() {
        if (null == sandboxGetter) {
            throw new IllegalStateException(getClass()+" hasn't been correctly initialised");
        }
        
        try {
            return ((Callable<Sandbox>)sandboxGetter).call();
        }
        catch (final Exception e) {
            throw new IllegalStateException("sandboxGetter produced an " +
                                           "exception, which is not expected!");
        }

    }
}
