package net.outlyer.plugins.utils;

/*
 * Copyright (c) 2008, Toni Corvera. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: LanguageExtensions.java 61 2008-07-01 00:27:28Z toni $

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import javax.script.ScriptException;
import net.outlyer.plugins.SandboxAccessorImpl;

/**
 * Extra facilities found in other languages or more complete
 * versions of available facilities.
 * Recommended name: lang
 */
public class LanguageExtensions extends SandboxAccessorImpl {
    /**
     * ScriptEngine's eval.
     * The built-in eval() appears to be less-powerful, in general they're
     * equivalent, but if the eval'ed string is obtainer from input
     * <code>eval()</code> appears to ignore it, while
     * <code>LanguageExtensions.eval()</code> does eval it.
     * @param code Code to evaluate
     * @return Eval's result
     */
    public Object eval(final String code) {
        try {
            return getSandbox().getCurrentEngine().eval(code);
        }
        catch (final ScriptException e) {
            // FIXME: Handle better
            System.err.println("Exception: " + e.getMessage());
        }
        return null;
    }

    public Object include(final String fileName) { // TODO: Support jar URIs
        try {
            return getSandbox().getCurrentEngine().eval(new FileReader(new File(fileName)));
        }
        catch (final FileNotFoundException e) {
            // FIXME: Handle better
            System.err.println("Failed to read "+fileName);
        }
        catch (final ScriptException e) {
            // FIXME: Handle better
            System.err.println("Exception: " + e.getMessage());
        }
        return null;        
    }
}
